@@ -74,23 +74,23 @@
    /* Put the tokens into the symbol table, so that GDB and other debuggers
       know about them.  */
    enum yytokentype {
+     END = 0,
      STRING = 258,
      NUM = 259,
      KEYWORD = 260,
      LISTOPEN = 261,
      LISTCLOSE = 262,
-     EOFF = 263,
-     ERROR = 264
+     ERROR = 263
    };
 #endif
 /* Tokens.  */
+#define END 0
 #define STRING 258
 #define NUM 259
 #define KEYWORD 260
 #define LISTOPEN 261
 #define LISTCLOSE 262
-#define EOFF 263
-#define ERROR 264
+#define ERROR 263
 
 
 
@@ -99,56 +99,60 @@
 
 
 
-/* 
+/*
    IGraph library.
    Copyright (C) 2009-2012  Gabor Csardi <csardi.gabor@gmail.com>
    334 Harvard st, Cambridge, MA, 02138 USA
-   
+
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
-   
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc.,  51 Franklin Street, Fifth Floor, Boston, MA 
+   Foundation, Inc.,  51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301 USA
 
 */
 
+#include "igraph_error.h"
+#include "igraph_memory.h"
+
+#include "io/gml-header.h"
+#include "io/gml-tree.h"
+#include "io/parsers/gml-parser.h"
+#include "io/parsers/gml-lexer.h"
+#include "io/parse_utils.h"
+#include "internal/hacks.h" /* strcasecmp & strndup */
+
 #include <stdio.h>
 #include <math.h>
 #include <string.h>
 
-#include "igraph_error.h"
-#include "igraph_memory.h"
-#include "config.h"
-
-#include "core/math.h"
-#include "io/gml-header.h"
-#include "io/gml-tree.h"
-#include "io/parsers/gml-parser.h"
-#include "io/parsers/gml-lexer.h"
-#include "internal/hacks.h" /* strcasecmp */
-
-int igraph_gml_yyerror(YYLTYPE* locp, igraph_i_gml_parsedata_t *context, 
+int igraph_gml_yyerror(YYLTYPE* locp, igraph_i_gml_parsedata_t *context,
                        const char *s);
-void igraph_i_gml_get_keyword(char *s, int len, void *res);
-void igraph_i_gml_get_string(char *s, int len, void *res);
-double igraph_i_gml_get_real(char *s, int len);
-igraph_gml_tree_t *igraph_i_gml_make_numeric(char* s, int len, double value);
-igraph_gml_tree_t *igraph_i_gml_make_numeric2(char* s, int len, 
-                                              char *v, int vlen);
-igraph_gml_tree_t *igraph_i_gml_make_string(char* s, int len, 
-                                            char *value, int valuelen);
-igraph_gml_tree_t *igraph_i_gml_make_list(char* s, int len, 
-                                          igraph_gml_tree_t *list);
-igraph_gml_tree_t *igraph_i_gml_merge(igraph_gml_tree_t *t1, igraph_gml_tree_t* t2);
+static igraph_error_t igraph_i_gml_get_keyword(const char *s, size_t len, char **res);
+static igraph_error_t igraph_i_gml_get_string(const char *s, size_t len, char **res);
+static igraph_error_t igraph_i_gml_make_numeric(const char *name,
+                                                int line,
+                                                igraph_real_t value,
+                                                igraph_gml_tree_t **tree);
+static igraph_error_t igraph_i_gml_make_string(const char *name,
+                                               int line,
+                                               char *value,
+                                               igraph_gml_tree_t **tree);
+static igraph_error_t igraph_i_gml_make_list(const char *name,
+                                             int line,
+                                             igraph_gml_tree_t *list,
+                                             igraph_gml_tree_t **tree);
+static igraph_error_t igraph_i_gml_make_empty(igraph_gml_tree_t **tree);
+static igraph_error_t igraph_i_gml_merge(igraph_gml_tree_t *t1, igraph_gml_tree_t* t2);
 
 #define scanner context->scanner
 #define USE(x) /*(x)*/
@@ -177,12 +181,9 @@ igraph_gml_tree_t *igraph_i_gml_merge(igraph_gml_tree_t *t1, igraph_gml_tree_t*
 typedef union YYSTYPE
 
 {
-   struct {
-      char *s;
-      int len;
-   } str;
-   void *tree;
-   double real;
+   char *str;
+   igraph_gml_tree_t *tree;
+   igraph_real_t real;
 }
 /* Line 193 of yacc.c.  */
 
@@ -427,20 +428,20 @@ union yyalloc
 /* YYFINAL -- State number of the termination state.  */
 #define YYFINAL  6
 /* YYLAST -- Last index in YYTABLE.  */
-#define YYLAST   14
+#define YYLAST   10
 
 /* YYNTOKENS -- Number of terminals.  */
-#define YYNTOKENS  10
+#define YYNTOKENS  9
 /* YYNNTS -- Number of nonterminals.  */
 #define YYNNTS  7
 /* YYNRULES -- Number of rules.  */
-#define YYNRULES  12
+#define YYNRULES  11
 /* YYNRULES -- Number of states.  */
-#define YYNSTATES  17
+#define YYNSTATES  15
 
 /* YYTRANSLATE(YYLEX) -- Bison symbol number corresponding to YYLEX.  */
 #define YYUNDEFTOK  2
-#define YYMAXUTOK   264
+#define YYMAXUTOK   263
 
 #define YYTRANSLATE(YYX)						\
   ((unsigned int) (YYX) <= YYMAXUTOK ? yytranslate[YYX] : YYUNDEFTOK)
@@ -474,7 +475,7 @@ static const yytype_uint8 yytranslate[] =
        2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
        2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
        2,     2,     2,     2,     2,     2,     1,     2,     3,     4,
-       5,     6,     7,     8,     9
+       5,     6,     7,     8
 };
 
 #if YYDEBUG
@@ -482,24 +483,23 @@ static const yytype_uint8 yytranslate[] =
    YYRHS.  */
 static const yytype_uint8 yyprhs[] =
 {
-       0,     0,     3,     5,     8,    10,    13,    16,    19,    24,
-      27,    29,    31
+       0,     0,     3,     5,     6,     8,    11,    14,    17,    22,
+      24,    26
 };
 
 /* YYRHS -- A `-1'-separated list of the rules' RHS.  */
 static const yytype_int8 yyrhs[] =
 {
-      11,     0,    -1,    12,    -1,    12,     8,    -1,    13,    -1,
-      12,    13,    -1,    14,    15,    -1,    14,    16,    -1,    14,
-       6,    12,     7,    -1,    14,    14,    -1,     5,    -1,     4,
-      -1,     3,    -1
+      10,     0,    -1,    11,    -1,    -1,    12,    -1,    11,    12,
+      -1,    13,    14,    -1,    13,    15,    -1,    13,     6,    11,
+       7,    -1,     5,    -1,     4,    -1,     3,    -1
 };
 
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const yytype_uint8 yyrline[] =
 {
-       0,   119,   119,   120,   123,   124,   126,   128,   130,   132,
-     136,   139,   142
+       0,   122,   122,   124,   125,   126,   128,   130,   132,   136,
+     139,   147
 };
 #endif
 
@@ -508,9 +508,9 @@ static const yytype_uint8 yyrline[] =
    First, the terminals, then, starting at YYNTOKENS, nonterminals.  */
 static const char *const yytname[] =
 {
-  "$end", "error", "$undefined", "STRING", "NUM", "KEYWORD", "LISTOPEN",
-  "LISTCLOSE", "EOFF", "ERROR", "$accept", "input", "list", "keyvalue",
-  "key", "num", "string", 0
+  "\"end of file\"", "error", "$undefined", "\"string\"", "\"number\"",
+  "\"keyword\"", "\"[\"", "\"]\"", "ERROR", "$accept", "input", "list",
+  "keyvalue", "key", "num", "string", 0
 };
 #endif
 
@@ -519,22 +519,22 @@ static const char *const yytname[] =
    token YYLEX-NUM.  */
 static const yytype_uint16 yytoknum[] =
 {
-       0,   256,   257,   258,   259,   260,   261,   262,   263,   264
+       0,   256,   257,   258,   259,   260,   261,   262,   263
 };
 # endif
 
 /* YYR1[YYN] -- Symbol number of symbol that rule YYN derives.  */
 static const yytype_uint8 yyr1[] =
 {
-       0,    10,    11,    11,    12,    12,    13,    13,    13,    13,
-      14,    15,    16
+       0,     9,    10,    11,    11,    11,    12,    12,    12,    13,
+      14,    15
 };
 
 /* YYR2[YYN] -- Number of symbols composing right hand side of rule YYN.  */
 static const yytype_uint8 yyr2[] =
 {
-       0,     2,     1,     2,     1,     2,     2,     2,     4,     2,
-       1,     1,     1
+       0,     2,     1,     0,     1,     2,     2,     2,     4,     1,
+       1,     1
 };
 
 /* YYDEFACT[STATE-NAME] -- Default rule to reduce with in state
@@ -542,14 +542,14 @@ static const yytype_uint8 yyr2[] =
    means the default is an error.  */
 static const yytype_uint8 yydefact[] =
 {
-       0,    10,     0,     2,     4,     0,     1,     3,     5,    12,
-      11,     0,     9,     6,     7,     0,     8
+       3,     9,     0,     2,     4,     0,     1,     5,    11,    10,
+       3,     6,     7,     0,     8
 };
 
 /* YYDEFGOTO[NTERM-NUM].  */
 static const yytype_int8 yydefgoto[] =
 {
-      -1,     2,     3,     4,     5,    13,    14
+      -1,     2,     3,     4,     5,    11,    12
 };
 
 /* YYPACT[STATE-NUM] -- Index in YYTABLE of the portion describing
@@ -557,14 +557,14 @@ static const yytype_int8 yydefgoto[] =
 #define YYPACT_NINF -4
 static const yytype_int8 yypact[] =
 {
-       1,    -4,    10,     0,    -4,    -2,    -4,    -4,    -4,    -4,
-      -4,     1,    -4,    -4,    -4,     2,    -4
+       1,    -4,     3,     1,    -4,    -2,    -4,    -4,    -4,    -4,
+       1,    -4,    -4,     0,    -4
 };
 
 /* YYPGOTO[NTERM-NUM].  */
 static const yytype_int8 yypgoto[] =
 {
-      -4,    -4,     3,    -3,     6,    -4,    -4
+      -4,    -4,    -1,    -3,    -4,    -4,    -4
 };
 
 /* YYTABLE[YYPACT[STATE-NUM]].  What to do in state STATE-NUM.  If
@@ -574,22 +574,22 @@ static const yytype_int8 yypgoto[] =
 #define YYTABLE_NINF -1
 static const yytype_uint8 yytable[] =
 {
-       8,     9,    10,     1,    11,     1,     1,     1,     7,    16,
-       6,    12,     8,     0,    15
+       7,     8,     9,     6,    10,     1,     1,    14,     0,    13,
+       7
 };
 
 static const yytype_int8 yycheck[] =
 {
-       3,     3,     4,     5,     6,     5,     5,     5,     8,     7,
-       0,     5,    15,    -1,    11
+       3,     3,     4,     0,     6,     5,     5,     7,    -1,    10,
+      13
 };
 
 /* YYSTOS[STATE-NUM] -- The (internal number of the) accessing
    symbol of state STATE-NUM.  */
 static const yytype_uint8 yystos[] =
 {
-       0,     5,    11,    12,    13,    14,     0,     8,    13,     3,
-       4,     6,    14,    15,    16,    12,     7
+       0,     5,    10,    11,    12,    13,     0,    12,     3,     4,
+       6,    14,    15,    11,     7
 };
 
 #define yyerrok		(yyerrstatus = 0)
@@ -1111,29 +1111,24 @@ yydestruct (yymsg, yytype, yyvaluep, yylocationp, context)
 
   switch (yytype)
     {
-      case 5: /* "KEYWORD" */
-
-	{ IGRAPH_FREE((yyvaluep->str).s); };
-
-	break;
-      case 12: /* "list" */
+      case 11: /* "list" */
 
 	{ igraph_gml_tree_destroy((yyvaluep->tree)); };
 
 	break;
-      case 13: /* "keyvalue" */
+      case 12: /* "keyvalue" */
 
 	{ igraph_gml_tree_destroy((yyvaluep->tree)); };
 
 	break;
-      case 14: /* "key" */
+      case 13: /* "key" */
 
-	{ IGRAPH_FREE((yyvaluep->str).s); };
+	{ free((yyvaluep->str)); };
 
 	break;
-      case 16: /* "string" */
+      case 15: /* "string" */
 
-	{ IGRAPH_FREE((yyvaluep->str).s); };
+	{ free((yyvaluep->str)); };
 
 	break;
 
@@ -1463,7 +1458,7 @@ yyreduce:
 
   case 3:
 
-    { context->tree=(yyvsp[(1) - (2)].tree); ;}
+    { IGRAPH_YY_CHECK(igraph_i_gml_make_empty(&(yyval.tree))); ;}
     break;
 
   case 4:
@@ -1473,47 +1468,47 @@ yyreduce:
 
   case 5:
 
-    { (yyval.tree)=igraph_i_gml_merge((yyvsp[(1) - (2)].tree), (yyvsp[(2) - (2)].tree)); ;}
+    { IGRAPH_YY_CHECK(igraph_i_gml_merge((yyvsp[(1) - (2)].tree), (yyvsp[(2) - (2)].tree))); (yyval.tree) = (yyvsp[(1) - (2)].tree); ;}
     break;
 
   case 6:
 
-    { (yyval.tree)=igraph_i_gml_make_numeric((yyvsp[(1) - (2)].str).s, (yyvsp[(1) - (2)].str).len, (yyvsp[(2) - (2)].real)); ;}
+    { IGRAPH_YY_CHECK(igraph_i_gml_make_numeric((yyvsp[(1) - (2)].str), (yylsp[(1) - (2)]).first_line, (yyvsp[(2) - (2)].real), &(yyval.tree))); ;}
     break;
 
   case 7:
 
-    { (yyval.tree)=igraph_i_gml_make_string((yyvsp[(1) - (2)].str).s, (yyvsp[(1) - (2)].str).len, (yyvsp[(2) - (2)].str).s, (yyvsp[(2) - (2)].str).len); ;}
+    { IGRAPH_YY_CHECK(igraph_i_gml_make_string((yyvsp[(1) - (2)].str), (yylsp[(1) - (2)]).first_line, (yyvsp[(2) - (2)].str), &(yyval.tree))); ;}
     break;
 
   case 8:
 
-    { (yyval.tree)=igraph_i_gml_make_list((yyvsp[(1) - (4)].str).s, (yyvsp[(1) - (4)].str).len, (yyvsp[(3) - (4)].tree)); ;}
+    { IGRAPH_YY_CHECK(igraph_i_gml_make_list((yyvsp[(1) - (4)].str), (yylsp[(1) - (4)]).first_line, (yyvsp[(3) - (4)].tree), &(yyval.tree))); ;}
     break;
 
   case 9:
 
-    { (yyval.tree)=igraph_i_gml_make_numeric2((yyvsp[(1) - (2)].str).s, (yyvsp[(1) - (2)].str).len, (yyvsp[(2) - (2)].str).s, (yyvsp[(2) - (2)].str).len); ;}
+    { IGRAPH_YY_CHECK(igraph_i_gml_get_keyword(igraph_gml_yyget_text(scanner),
+                               igraph_gml_yyget_leng(scanner),
+                               &(yyval.str))); USE((yyvsp[(1) - (1)].str)); ;}
     break;
 
   case 10:
 
-    { igraph_i_gml_get_keyword(igraph_gml_yyget_text(scanner), 
-                                        igraph_gml_yyget_leng(scanner), 
-                                        &(yyval.str)); USE((yyvsp[(1) - (1)].str)); ;}
+    {
+    igraph_real_t val;
+    IGRAPH_YY_CHECK(igraph_i_parse_real(igraph_gml_yyget_text(scanner),
+                                        igraph_gml_yyget_leng(scanner),
+                                        &val));
+    (yyval.real)=val;
+;}
     break;
 
   case 11:
 
-    { (yyval.real)=igraph_i_gml_get_real(igraph_gml_yyget_text(scanner), 
-                                     igraph_gml_yyget_leng(scanner)); ;}
-    break;
-
-  case 12:
-
-    { igraph_i_gml_get_string(igraph_gml_yyget_text(scanner), 
-                                         igraph_gml_yyget_leng(scanner), 
-                                         &(yyval.str)); ;}
+    { IGRAPH_YY_CHECK(igraph_i_gml_get_string(igraph_gml_yyget_text(scanner),
+                                         igraph_gml_yyget_leng(scanner),
+                                         &(yyval.str))); ;}
     break;
 
 
@@ -1741,154 +1736,121 @@ yyreturn:
 
 
 
-int igraph_gml_yyerror(YYLTYPE* locp, igraph_i_gml_parsedata_t *context, 
+int igraph_gml_yyerror(YYLTYPE* locp, igraph_i_gml_parsedata_t *context,
                        const char *s) {
-  snprintf(context->errmsg, sizeof(context->errmsg)/sizeof(char)-1, 
-           "Parse error in GML file, line %i (%s)", 
+  snprintf(context->errmsg, sizeof(context->errmsg)/sizeof(char)-1,
+           "Parse error in GML file, line %i (%s)",
            locp->first_line, s);
   return 0;
 }
 
-void igraph_i_gml_get_keyword(char *s, int len, void *res) {
-  struct { char *s; int len; } *p=res;
-  p->s=IGRAPH_CALLOC(len+1, char);
-  if (!p->s) { 
-    igraph_error("Cannot read GML file", IGRAPH_FILE_BASENAME, __LINE__, IGRAPH_PARSEERROR);
+static igraph_error_t igraph_i_gml_get_keyword(const char *s, size_t len, char **res) {
+  *res = strndup(s, len);
+  if (! *res) {
+    IGRAPH_ERROR("Cannot read GML file.", IGRAPH_ENOMEM); /* LCOV_EXCL_LINE */
   }
-  memcpy(p->s, s, sizeof(char)*len);
-  p->s[len]='\0';
-  p->len=len;
+  return IGRAPH_SUCCESS;
 }
 
-void igraph_i_gml_get_string(char *s, int len, void *res) {
-  struct { char *s; int len; } *p=res;
-  p->s=IGRAPH_CALLOC(len-1, char);
-  if (!p->s) { 
-    igraph_error("Cannot read GML file", IGRAPH_FILE_BASENAME, __LINE__, IGRAPH_PARSEERROR);
+static igraph_error_t igraph_i_gml_get_string(const char *s, size_t len, char **res) {
+  *res = strndup(s+1, len-2);
+  if (! *res) {
+    IGRAPH_ERROR("Cannot read GML file.", IGRAPH_ENOMEM); /* LCOV_EXCL_LINE */
   }
-  memcpy(p->s, s+1, sizeof(char)*(len-2));
-  p->s[len-2]='\0';
-  p->len=len-2;
+  return IGRAPH_SUCCESS;
 }
 
-double igraph_i_gml_get_real(char *s, int len) {
-  igraph_real_t num;
-  char tmp=s[len];
-  s[len]='\0';
-  sscanf(s, "%lf", &num);
-  s[len]=tmp;
-  return num;
-} 
+static igraph_error_t igraph_i_gml_make_numeric(const char *name,
+                                                int line,
+                                                igraph_real_t value,
+                                                igraph_gml_tree_t **tree) {
 
-igraph_gml_tree_t *igraph_i_gml_make_numeric(char* s, int len, double value) {
   igraph_gml_tree_t *t = IGRAPH_CALLOC(1, igraph_gml_tree_t);
-
-  if (!t) { 
-    igraph_error("Cannot build GML tree", IGRAPH_FILE_BASENAME, __LINE__, IGRAPH_ENOMEM);
-    return 0;
+  if (!t) {
+    IGRAPH_ERROR("Cannot build GML tree.", IGRAPH_ENOMEM); /* LCOV_EXCL_LINE */
   }
+  IGRAPH_FINALLY(igraph_free, t);
 
-  if (floor(value)==value) {
-    if (igraph_gml_tree_init_integer(t, s, len, value)) {
-          free(t);
-          return 0;
-        }
+  /* The GML spec only requires support for 32-bit signed integers.
+   * We treat anything out of that range as real. These values end
+   * up as igraph_real_t anyway, as igraph does not currently support
+   * integer-typed attributes. */
+  if (floor(value) == value && value >= INT32_MIN && value <= INT32_MAX) {
+    IGRAPH_CHECK(igraph_gml_tree_init_integer(t, name, line, value));
   } else {
-    if (igraph_gml_tree_init_real(t, s, len, value)) {
-      free(t);
-          return 0;
-        }
+    IGRAPH_CHECK(igraph_gml_tree_init_real(t, name, line, value));
   }
-  
-  return t;
+
+  *tree = t;
+  IGRAPH_FINALLY_CLEAN(1); /* t */
+
+  return IGRAPH_SUCCESS;
 }
 
-igraph_gml_tree_t *igraph_i_gml_make_numeric2(char* s, int len, 
-                                              char* v, int vlen) {
+static igraph_error_t igraph_i_gml_make_string(const char *name,
+                                               int line,
+                                               char *value,
+                                               igraph_gml_tree_t **tree) {
+
   igraph_gml_tree_t *t = IGRAPH_CALLOC(1, igraph_gml_tree_t);
-  char tmp = v[vlen];
-
-  if (!t) { 
-    igraph_error("Cannot build GML tree", IGRAPH_FILE_BASENAME, __LINE__, IGRAPH_ENOMEM);
-    return 0;
-  }
-
-  v[vlen]='\0';
-
-  /* if v == "inf" or v == "nan", the newly created tree node will take ownership
-   * of s. If the creation fails, we need to free s and v as well in order not
-   * to leak memory */
-  if (strcasecmp(v, "inf")) {
-    if (igraph_gml_tree_init_real(t, s, len, IGRAPH_INFINITY)) {
-      free(t);
-      t = 0;
-    }
-  } else if (strcasecmp(v, "nan")) {
-    if (igraph_gml_tree_init_real(t, s, len, IGRAPH_NAN)) {
-      free(t);
-      t = 0;
-    }
-  } else {
-    igraph_error("Parse error", IGRAPH_FILE_BASENAME, __LINE__, IGRAPH_PARSEERROR);
-    free(t);
-    t = 0;
-  }
-
-  v[vlen]=tmp;
-  free(v);
-
-  if (t == 0) {
-    /* no new tree node was created so s has no owner any more */
-    free(s);
-  }
-
-  return t;
-}
-
-igraph_gml_tree_t *igraph_i_gml_make_string(char* s, int len, 
-                                            char *value, int valuelen) {
-  igraph_gml_tree_t *t = IGRAPH_CALLOC(1, igraph_gml_tree_t);
-
-  if (!t) { 
-    igraph_error("Cannot build GML tree", IGRAPH_FILE_BASENAME, __LINE__, IGRAPH_ENOMEM);
-    return 0;
+  if (!t) {
+    IGRAPH_ERROR("Cannot build GML tree.", IGRAPH_ENOMEM); /* LCOV_EXCL_LINE */
   }
+  IGRAPH_FINALLY(igraph_free, t);
 
   /* if igraph_gml_tree_init_string succeeds, the newly created tree node takes
    * ownership of 'value'. If it fails, we need to free 'value' ourselves in order
    * not to leak memory */
-  if (igraph_gml_tree_init_string(t, s, len, value, valuelen)) {
-    free(t);
-    free(value);
-    t = 0;
-  }
+  IGRAPH_FINALLY(igraph_free, value);
+  IGRAPH_CHECK(igraph_gml_tree_init_string(t, name, line, value));
 
-  return t;
+  IGRAPH_FINALLY_CLEAN(1); /* value */
+
+  *tree = t;
+  IGRAPH_FINALLY_CLEAN(1); /* t */
+
+  return IGRAPH_SUCCESS;
 }
 
-igraph_gml_tree_t *igraph_i_gml_make_list(char* s, int len, 
-                                          igraph_gml_tree_t *list) {
-  
-  igraph_gml_tree_t *t=IGRAPH_CALLOC(1, igraph_gml_tree_t);
+static igraph_error_t igraph_i_gml_make_list(const char *name,
+                                             int line,
+                                             igraph_gml_tree_t *list,
+                                             igraph_gml_tree_t **tree) {
 
-  if (!t) { 
-    igraph_error("Cannot build GML tree", IGRAPH_FILE_BASENAME, __LINE__, IGRAPH_ENOMEM);
-    return 0;
+  igraph_gml_tree_t *t = IGRAPH_CALLOC(1, igraph_gml_tree_t);
+  if (!t) {
+    IGRAPH_ERROR("Cannot build GML tree.", IGRAPH_ENOMEM); /* LCOV_EXCL_LINE */
   }
+  IGRAPH_FINALLY(igraph_free, t);
 
-  if (igraph_gml_tree_init_tree(t, s, len, list)) {
-    free(t);
-    return 0;
-  }
+  IGRAPH_CHECK(igraph_gml_tree_init_tree(t, name, line, list));
 
-  return t;
+  *tree = t;
+  IGRAPH_FINALLY_CLEAN(1); /* t */
+
+  return IGRAPH_SUCCESS;
 }
 
-igraph_gml_tree_t *igraph_i_gml_merge(igraph_gml_tree_t *t1, igraph_gml_tree_t* t2) {
+static igraph_error_t igraph_i_gml_make_empty(igraph_gml_tree_t **tree) {
+    igraph_gml_tree_t *t = IGRAPH_CALLOC(1, igraph_gml_tree_t);
+    if (!t) {
+      IGRAPH_ERROR("Cannot build GML tree.", IGRAPH_ENOMEM); /* LCOV_EXCL_LINE */
+    }
+    IGRAPH_FINALLY(igraph_free, t);
 
-  igraph_gml_tree_mergedest(t1, t2);
+    IGRAPH_CHECK(igraph_gml_tree_init_empty(t));
+
+    *tree = t;
+    IGRAPH_FINALLY_CLEAN(1); /* t */
+
+    return IGRAPH_SUCCESS;
+}
+
+static igraph_error_t igraph_i_gml_merge(igraph_gml_tree_t *t1, igraph_gml_tree_t* t2) {
+
+  IGRAPH_CHECK(igraph_gml_tree_mergedest(t1, t2));
   IGRAPH_FREE(t2);
 
-  return t1;
+  return IGRAPH_SUCCESS;
 }
 
